/*  File: mod-dyB.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   Deductor headers  */
#include <deductor.h>
#include <proc-dyjets.h>

/*   Analyser headers  */
#include "analyser-BpT.h"


/*   deductor and standard namespaces   */
using namespace duct;
using namespace std;

/*   I think this is the safest way, if we use C linking for the global variables.
 *   May be it is not necessary. I think global variables on the global namespace 
 *   has C linkage by default.
 */
extern "C" {
  
  /*
   *   This the user defined main program. It is more flexible and simple
   *   this way. User can start more then one process simultanously with many
   *   analyser each.
   */
  std::function<void(const char *, const char *)>
  main_calc = [=](const char *dir, const char *rname) -> void
  {

    /*  Using an already created kernel. */
    deductor<hhc> kernel("LHC-8TeV");
    
    /*  Shower cutoff is given in the transverse momentum of the splitting.
     *  This overrides the kernel setting (kTmin, see above) if it is bigger.
     *  This can be overidden in unsigned pid = kernel("Jets", p, hard, Jets) below.
     */
    kernel.kT_cutoff(1.0_GeV);
    kernel.max_color_suppression(0u);
    
    /*   Drell-Yan Z-boson pT distribution    */
    {
      /*   Creating the hard process. Actually we don't create the hard process here.
       *   The kernel has its own model descriptor and it know the c.m. energy (Ecm).
       *   The model and the Ecm have to be matched in the kernel and in the hard process.
       *   Here we create a function object that can create the desired hard process.
       */
      auto hard = [=](const model *mdl, double Ecm) -> dyjets
      {
        double mL = 400.0_GeV, mH = 5.0_TeV;
        int proton = 2212, electron = 11, bquark = 5;
        
        return dyjets(mdl, Ecm, mL, mH, proton, proton, {electron}, {bquark});
      };
      
      /*   Creating the analyser object. 
       *   Every analyser has to be dynamically allocated!
       */
      unsigned long nsave = 100000UL;   // Backs up each time nsave events are calculated.
      unsigned long nevent = 0UL;       // 0 events means an infinite loop.
      
      UserBpT *BpT = new UserBpT;
      BpT->param(dir, rname, "BpT", nsave, nevent);
      
      /*  Create the shower process and start the calculation and returns the process ID.   */
      unsigned pid = kernel("Drell-Yan Bbar pT", hard, BpT);
      
      /*  Here we can amouse ourself with fancy messages like these...  */
      std::cout<<"Process has been created and started.\n";
      std::cout<<"  Process Name  : "<<kernel.name(pid)<<"\n";
      std::cout<<"  Process ID    : "<<pid<<"\n"<<std::endl;
    }
    
  };
  
  
  /*   This is the user defined main routine for deductor --add ...
   *   It is very simple, since it just returns the list of all analyser.
   */
  main_add_list main_add = {to_main_add<UserBpT>("BpT")};
  
}  // extern "C"



