/*  File: analysert.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-bfkl.h"

/*   other includes   */




void UserBFKL::initfunc()
{
  /*  Create histograms to calculate the average number of jets vs DY
   *  Here DY is difference in rapidities between jets with the largest |rapidity|
   *  We will require a minimum PT for these jets.
   */
  unsigned int nDYbins = 28;
  double DYmin = 0.0, DYmax = 8.0;
  
  UserRatio::phys(1, "Average number of jets", spacing<hist1d>::linear(nDYbins, DYmin, DYmax));
  
  cout << "Analyzer UserBFKL is using jet algorithm "
  << jetDef.jet_algorithm()
  << " with R = " << jetDef.R() << endl;

  /*   We want to generate some LaTeX     */
  UserRatio::enable_plot(1);
  
  /*   The first plot is the ratio   */
  ltxfigure pTplot;
  
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$\\langle N\\rangle(\\Delta y)$},"
  "xmin=0, xmax=8, ymin=2, ymax=3";
  
  UserRatio::add_figure(1, pTplot);
  
  /*    The numerator and the denominator in one figure  */
  pTplot.axis = "xlabel={$\\Delta y$}, ylabel={$\\displaystyle\\int n_{\\rm jets}^\\beta\\, d\\sigma(\\Delta y)$\; [nb]},"
  "xmin=0, xmax=8, ymin=0";
  
  pTplot.add_plot2D("xm", "val2", "blue");
  pTplot.add_plot2D("xm", "val3", "green!60!black");
 
  UserRatio::add_figure(1, pTplot);
}



void UserBFKL::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double wb = sb.weight*color_weight(sb)*tonb;
  
  /* preparing the momenta for FastJet */
  make_pseudojets(theparticles, p);
  
  /*   Clustering the partons using FastJet */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets (pT > 30GeV) */
  incJets = clustSeq.inclusive_jets(30.0_GeV);
  
  /*    Select the jets with |y| < 4.5.  */
  OKjets.resize(0);
  
  for(int n = 0; n < incJets.size(); n++)
    if(std::abs(incJets[n].rap()) < 4.5)
      OKjets.push_back(incJets[n]);
  
  
  /*   Needs at least 2 jets  */
  unsigned int nj = OKjets.size();
  if(nj < 2) return;
  
  /*  sorting them by rapidity  */
  std::sort(OKjets.begin(), OKjets.end(),
            [](const PseudoJet& a, const PseudoJet& b)->bool {
              return a.rap() < b.rap();
            });
  
  /*   most backward and most forward jets   */
  const PseudoJet &jetF = OKjets.back(), &jetB = OKjets.front();
  
  /* Average pT of the most forward and backward has to be at least 60 GeV  */
  if(jetF.perp()+jetB.perp() > 120.0_GeV)
    physfill(1, {{wb*nj, wb}}, dirac(), std::abs(jetF.rap()-jetB.rap()));
}

