/*  File: analyser-xperp.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-xperp.h"

/*   other includes   */


void UserXperp::initfunc()
{
  /*  Create histograms to calculate the one jet cross section vs DY
   *  Here DY is difference in rapidities between jets with the largest |rapidity|
   *  We will require a minimum PT for these jets.
   */
  unsigned int Nxperpbins = 40;
  double xperpmax = 2.0, xperpmin = -2.0;
  bool enable_norm = true;

  phys(1, "xperp distribution of the soft jet.",
       spacing<hist1d>::linear(Nxperpbins, xperpmin, xperpmax),
       enable_norm);
  
  cout<< "UserXperp is using jet algorithm "
  << jetDef.jet_algorithm()
  << " with R = " << jetDef.R() << endl;
}


inline double deltaPhi(const PseudoJet& pi, const PseudoJet& pj)
{
  double dphi = pj.phi() - pi.phi();
  
  if(M_PI < dphi)  dphi -= 2.0*M_PI;
  if(dphi < -M_PI) dphi += 2.0*M_PI;
  
  if(M_PI < dphi || dphi < -M_PI)
    cerr<<"Oops, wrong range for dphi."<<endl;
  
  return dphi;
}



void UserXperp::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*  Loop over event record to calculate ET (|y| < 3.6)
   *  in order to decide if we want this event.
   */
  double ET = 0.0;
  
  for(unsigned  i = 1; i <= p.number_of<qcd_out>(); ++i)
    if(abs(p[i].momentum.rapidity()) < 3.6)
      ET += p[i].momentum.perp();
  
  if(ET < 400.0) return;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  double w = sb.weight*color_weight(sb)*tonb;
  
  
  /*   Converting deductor event to FastJet format   */
  make_pseudojets(theparticles,p);
  
  /*   Run fastjet algorithm   */
  ClusterSequence clustSeq(theparticles, jetDef);
  
  /*   Extract inclusive jets sorted by pT (above 4GeV)  */
  incJets = clustSeq.inclusive_jets(4.0_GeV);
    
  /*   Fill the historgram   */
  constexpr double pTmin = 50.0_GeV;
  
  for(unsigned int i = 0; i < incJets.size()-1; ++i)
    for(unsigned int j = i+1; j < incJets.size(); ++j)
      if(incJets[j].pt() > pTmin && incJets[i].pt() > pTmin)
      {
        double rapji = incJets[j].rap() - incJets[i].rap();
        double phiji = deltaPhi(incJets[i], incJets[j]);
        double theta2ji = rapji*rapji + phiji*phiji;
        
        /*   theta cut  |thetaij- 0.5| < 0.1    */
        if(std::abs(sqrt(theta2ji)- 0.5) < 0.1)
          for(unsigned int k = 0; k < incJets.size(); k++)
            if(k != i && k != j)
            {
              double rapki = incJets[k].rap() - incJets[i].rap();
              double phiki = deltaPhi(incJets[i], incJets[k]);
              
              double xparallel = (rapki*rapji + phiki*phiji)/theta2ji;
              double xperp = (phiki*rapji - rapki*phiji)/theta2ji;
              
              /*   cuts: |xperp| < 2  & 1/3 < xparallel < 2/3   */
              if(1.0/3.0 < xparallel && xparallel < 2.0/3.0 && std::abs(xperp) < 2.0) {
                normfill(1, w);
                physfill(1, w, dirac(), xperp);
              }
            }
      }
}

