/*  File: mod-dy.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */


/*   Deductor headers  */
#include <deductor.h>
#include <proc-hhcjets.h>
#include <proc-hhcjets-lc.h>

/*   Analyser headers  */
#include "analyser-jeppe.h"
#include "analyser-cms.h"


/*   deductor and standard namespaces   */
using namespace duct;
using namespace std;


/*   I think this is the safest way, if we use C linking for the global variables.
 *   May be it is not necessary. I think global variables on the global namespace 
 *   has C linkage by default.
 */
extern "C" {
  
  /*
   *   This the user defined main program. It is more flexible and simple
   *   this way. User can start more then one process simultanously with many
   *   analyser each.
   */
  std::function<void(const char *, const char *)>
  main_calc = [=](const char *dir, const char *rname) -> void
  {
    /*  Using a previously creating the kernel   */
    deductor<hhc> kernel("LHC-7TeV");       // For BFKL. Use Ecm = 7.0_TeV;
    
    /*   Set the default parameters for the kernel.
     This can be overidden in unsigned pid = kernel("Jets", p, hard, Jets) below.*/
    kernel.kT_cutoff(1.0_GeV);
    kernel.max_color_suppression(0u);
    
    /*   Process for BFKL study     */
    {
      /*   Creating the hard process. Actually we don't create the hard process here.
       *   The kernel has its own model descriptor and it know the c.m. energy (Ecm).
       *   The model and the Ecm have to be matched in the kernel and in the hard process.
       *   Here we create a function object that can create the desired hard process.
       */
      auto hard = [=](const model *mdl, double Ecm) -> hhcjets_lc
      {
        int proton = 2212;
        auto pTlist = {15.0, 40.0, 100.0, 200.0, 300.0, 400.0, 600.0, 800.0};
        
        return hhcjets_lc(mdl, Ecm, pTlist, proton, proton);
      };

      
      /*   Creating the analyser object.
       *   Every analyser has to be dynamically allocated!
       */
      UserJeppe *Jeppe = new UserJeppe;
      UserCMS *CMS = new UserCMS;

      Jeppe->param(dir, rname, "Jeppe", 10000UL);
      CMS->param(dir, rname, "CMS", 10000UL);
      
      /*  Creates the shower process, start the calculation and returns the process ID.   */
      // Or {user1,user2,user3}
      unsigned pid = kernel("Jets", hard, {Jeppe, CMS});
      
      /*  Here we can amuse ourself with fancy messages like these...  */
      std::cout<<"Process \'"<<kernel.name(pid)<<"\' has been created and started with process id, pid = "<<pid<<".\n";
    }
    
  };
  
  
  /*
   *   This is the user defined main routine for deductor --add ...
   *   It is very simple, since it just a list of all analyser.
   */
  main_add_list main_add = {to_main_add<UserJeppe>("Jeppe"), to_main_add<UserCMS>("CMS")};
  
}



